package com.example.playertime;

import com.google.gson.*;
import com.mojang.authlib.GameProfile;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.network.ServerPlayerEntity;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class PlayerTimeTracker {
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();
    private final MinecraftServer server;
    private final Path dataFile;
    private final ExecutorService executor = Executors.newSingleThreadExecutor();
    private final Map<UUID, PlayerTimeData> playerData = new ConcurrentHashMap<>();

    public PlayerTimeTracker(MinecraftServer server) {
        this.server = server;
        this.dataFile = server.getRunDirectory().resolve("player_time_data.json");
        loadData();
    }

    // 没用了，但是先保留吧
    // private boolean isWhitelisted(String playerName) {
    //     MinecraftServer server = this.server;
    //     if (server == null) return false;
    //
    //     return server.getPlayerManager().getWhitelist()
    //             .isAllowed(server.getUserCache().findByName(playerName).orElse(null));
    // }


    public void onPlayerJoin(ServerPlayerEntity player) {
        PlayerTimeData data = playerData.computeIfAbsent(player.getUuid(), uuid -> new PlayerTimeData());
        data.lastLogin = Instant.now().getEpochSecond();
        PlayerTimeMod.LOGGER.info("[在线时间] 玩家 {} 加入, 开始计时", player.getName().getString());
    }

    public void onPlayerLeave(ServerPlayerEntity player) {
        PlayerTimeData data = playerData.get(player.getUuid());
        if (data != null && data.lastLogin > 0) {
            long now = Instant.now().getEpochSecond();
            long sessionTime = now - data.lastLogin;

            if (sessionTime > 0) {
                data.totalTime += sessionTime;

                // 维护30天滚动窗口
                data.rolling30Days.addPlayTime(now, sessionTime);
                data.rolling7Days.addPlayTime(now, sessionTime);

                PlayerTimeMod.LOGGER.info("[在线时间] 玩家 {} 离开, 会话时间 {} 秒, 总计时间 {}",
                        player.getName().getString(), sessionTime, data.totalTime);
            } else {
                PlayerTimeMod.LOGGER.warn("[在线时间] 玩家 {} 离开, 但计算出的会话时间不正常 ({}), 可能是由于时间同步问题或快速重新连接导致",
                        player.getName().getString(), sessionTime);
            }

            data.lastLogin = 0;
            saveAsync(player.getUuid());
        } else {
            PlayerTimeMod.LOGGER.warn("[在线时间] 玩家 {} 已离开，但上次登录时间为0或数据不存在，跳过会话时间计算。", player.getName().getString());
        }
    }

    public PlayerTimeStats getPlayerStats(UUID uuid) {
        PlayerTimeData data = playerData.get(uuid);
        if (data == null) {
            return null;
        }

        long now = Instant.now().getEpochSecond();
        PlayerTimeStats stats = new PlayerTimeStats();
        stats.totalTime = data.totalTime;

        // 检查玩家是否当前在线，只在在线时才计算
        if (data.lastLogin > 0 && server.getPlayerManager().getPlayer(uuid) != null) {
            long currentSessionTime = now - data.lastLogin;
            if (currentSessionTime > 0) {
                stats.totalTime += currentSessionTime;
            }
        }

        stats.last30Days = data.rolling30Days.getTotalTime(now);
        stats.last7Days = data.rolling7Days.getTotalTime(now);

        return stats;
    }

    public Map<String, String> getWhitelistedPlayerStats() {
        Map<String, String> stats = new LinkedHashMap<>();
        long now = Instant.now().getEpochSecond();

        // 获取白名单玩家UUID集合
        Set<UUID> whitelistUuids = new HashSet<>();
        for (String name : server.getPlayerManager().getWhitelist().getNames()) {
            server.getUserCache().findByName(name).ifPresent(profile -> {
                whitelistUuids.add(profile.getId());
            });
        }

        // 遍历所有已记录玩家数据
        playerData.forEach((uuid, data) -> {
            // 只处理白名单玩家
            if (whitelistUuids.contains(uuid)) {
                String playerName = getPlayerName(uuid);
                long totalTime = data.totalTime;

                // 检查玩家是否当前在线，只在在线时才计算
                if (data.lastLogin > 0 && server.getPlayerManager().getPlayer(uuid) != null) {
                    long currentSessionTime = now - data.lastLogin;
                    if (currentSessionTime > 0) {
                        totalTime += currentSessionTime;
                    }
                }

                long last30DaysTime = data.rolling30Days.getTotalTime(now);
                long last7DaysTime = data.rolling7Days.getTotalTime(now);
                stats.put(playerName, PlayerTimeMod.getLocalizationManager().getString(
                        "playertime.stats.format",
                        formatTime(totalTime),
                        formatTime(last30DaysTime),
                        formatTime(last7DaysTime)
                ));
            }
        });

        return stats;
    }

    public String getPlayerName(UUID uuid) {
        ServerPlayerEntity player = server.getPlayerManager().getPlayer(uuid);
        if (player != null) {
            return player.getName().getString();
        }

        Optional<GameProfile> profile = server.getUserCache().getByUuid(uuid);
        if (profile.isPresent()) {
            return profile.get().getName();
        }

        return "Unknown";
    }

    private void loadData() {
        if (!Files.exists(dataFile)) {
            PlayerTimeMod.LOGGER.info("[在线时间] 数据文件未找到，跳过加载");
            return;
        }

        try (Reader reader = Files.newBufferedReader(dataFile)) {
            JsonElement jsonElement = JsonParser.parseReader(reader);

            if (jsonElement == null || !jsonElement.isJsonObject()) {
                PlayerTimeMod.LOGGER.warn("[在线时间] 数据文件为空或格式错误，跳过加载");
                return;
            }

            JsonObject root = jsonElement.getAsJsonObject();
            int resetCount = 0;

            for (Map.Entry<String, JsonElement> entry : root.entrySet()) {
                try {
                    UUID uuid = UUID.fromString(entry.getKey());
                    PlayerTimeData data = GSON.fromJson(entry.getValue(), PlayerTimeData.class);

                    if (data.lastLogin > 0) {
                        PlayerTimeMod.LOGGER.warn(
                                "[在线时间] 在数据加载过程中发现玩家{}（UUID：{}）的最后登录时间大于0（{}）。将其重置为0。",
                                getPlayerName(uuid), uuid, data.lastLogin
                        );
                        data.lastLogin = 0;
                        resetCount++;
                    }

                    playerData.put(uuid, data);
                } catch (IllegalArgumentException e) {
                    PlayerTimeMod.LOGGER.error("[在线时间] I数据文件中的 UUID 格式无效: " + entry.getKey(), e);
                } catch (JsonParseException e) {
                    PlayerTimeMod.LOGGER.error("[在线时间] 解析玩家数据失败(UUID: " + entry.getKey() + ")", e);
                }
            }
            PlayerTimeMod.LOGGER.info("[在线时间] 成功加载了 {} 名玩家的数据，重置了 {} 名玩家的上次登录时间", playerData.size(), resetCount);

        } catch (IOException e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法读取玩家在线时间数据文件", e);
        } catch (JsonParseException e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 玩家在线时间数据文件格式错误", e);
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 加载玩家在线时间数据时发生未知错误", e);
        }
    }

    public void saveAll() {
        PlayerTimeMod.LOGGER.info("[在线时间] 开始保存所有玩家数据...");
        JsonObject root = new JsonObject();
        playerData.forEach((uuid, data) -> {
            if (data.lastLogin > 0) {
                long now = Instant.now().getEpochSecond();
                long sessionTime = now - data.lastLogin;
                if (sessionTime > 0) {
                    data.totalTime += sessionTime;
                    data.rolling30Days.addPlayTime(now, sessionTime);
                    data.rolling7Days.addPlayTime(now, sessionTime);
                    PlayerTimeMod.LOGGER.debug("[在线时间] 累积当前会话时间 {} 秒，保存期间玩家 {}", sessionTime, getPlayerName(uuid));
                }
                data.lastLogin = 0;
            }
            root.add(uuid.toString(), GSON.toJsonTree(data));
        });

        try (Writer writer = Files.newBufferedWriter(dataFile)) {
            GSON.toJson(root, writer);
            PlayerTimeMod.LOGGER.info("[在线时间] 所有玩家数据已成功保存");
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法保存所有玩家在线时间数据", e);
        }
    }

    private void saveAsync(UUID uuid) {
        executor.execute(() -> {
            PlayerTimeData data = playerData.get(uuid);
            if (data == null) {
                return;
            }

            JsonObject root;
            try {
                if (Files.exists(dataFile)) {
                    try (Reader reader = Files.newBufferedReader(dataFile)) {
                        JsonElement jsonElement = JsonParser.parseReader(reader);
                        if (jsonElement != null && jsonElement.isJsonObject()) {
                            root = jsonElement.getAsJsonObject();
                        } else {
                            root = new JsonObject();
                        }
                    }
                } else {
                    root = new JsonObject();
                }
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 在异步保存期间无法读取数据文件，正在创建新对象", e);
                root = new JsonObject();
            }

            root.add(uuid.toString(), GSON.toJsonTree(data));

            try (Writer writer = Files.newBufferedWriter(dataFile)) {
                GSON.toJson(root, writer);
                // PlayerTimeMod.LOGGER.debug("[在线时间] Async save successful for player {}", getPlayerName(uuid));
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 无法异步保存玩家的在线时间数据 " + getPlayerName(uuid) + " (UUID: " + uuid + ")", e);
            }
        });
    }

    public static String formatTime(long seconds) {
        if (seconds < 0) seconds = 0;
        long hours = seconds / 3600;
        long minutes = (seconds % 3600) / 60;
        return String.format("%dh %02dm", hours, minutes);
    }

    public static class PlayerTimeData {
        long totalTime = 0;
        long lastLogin = 0;
        RollingTimeWindow rolling30Days = new RollingTimeWindow(30);
        RollingTimeWindow rolling7Days = new RollingTimeWindow(7);
    }

    public static class PlayerTimeStats {
        public long totalTime;
        public long last30Days;
        public long last7Days;
    }

    private static class RollingTimeWindow {
        private final int days;
        private final List<TimeEntry> entries = new LinkedList<>();

        public RollingTimeWindow(int days) {
            this.days = days;
        }

        public void addPlayTime(long timestamp, long seconds) {
            if (seconds <= 0) return;
            entries.add(new TimeEntry(timestamp, seconds));
            cleanUp(timestamp);
        }

        public long getTotalTime(long currentTime) {
            cleanUp(currentTime);
            return entries.stream().mapToLong(e -> e.seconds).sum();
        }

        private void cleanUp(long currentTime) {
            long cutoff = currentTime - (days * 24 * 3600L);
            while (!entries.isEmpty() && entries.get(0).timestamp < cutoff) {
                entries.remove(0);
            }
        }

        private static class TimeEntry {
            final long timestamp;
            final long seconds;

            TimeEntry(long timestamp, long seconds) {
                this.timestamp = timestamp;
                this.seconds = seconds;
            }
        }
    }

    public Map<UUID, PlayerTimeData> getPlayerData() {
        return Collections.unmodifiableMap(playerData);
    }

    public Path getDataFile() {
        return this.dataFile;
    }

}
